using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

using TAPI;
using Terraria;

namespace CribMod.Projectiles
{
    public class CrystalCometSpear : ModProjectile
    {
        public override void DealtNPC(NPC n, int hitDir, int dmgDealt, float knockback, bool crit)
        {
            n.AddBuff(44, 900); 
        }
        
		public override void PostKill()
        {
			Main.PlaySound(-1, (int)projectile.position.X, (int)projectile.position.Y, 1);
                {
                    int dustID = Dust.NewDust(projectile.position, projectile.width, projectile.height, 67, Main.rand.Next(0, 0) * 1f, Main.rand.Next(0, 0) * 1f, 90, Color.White, 2.5f);
                    Main.dust[dustID].noGravity = true;
                }
	}

	public override void AI()	
		{
			Player plr = Main.player[projectile.owner];
			AISpear(projectile, ref projectile.ai, plr.Center, plr.direction, plr.itemAnimation, plr.itemAnimationMax, 3f, 6f, 6.8f, false);
			if(projectile.direction == 1){ projectile.rotation -= (float)(Math.PI / 2f); }
		}

		public override bool PreDraw(SpriteBatch sb)
		{
			DrawProjectileSpear(sb, Main.projectileTexture[projectile.type], projectile, null, 0f, 0f);
			return false;
		}

		#region Non-hook stuff
		//spear AI
		public static void AISpear(Projectile p, ref float[] ai, Vector2 center, int ownerDirection, int itemAnimation, int itemAnimationMax, float initialSpeed = 3f, float moveOutward = 1.4f, float moveInward = 1.6f, bool overrideKill = false)
		{
			p.direction = ownerDirection;
			p.Center = center;
			if (ai[0] == 0f) { ai[0] = initialSpeed; p.netUpdate = true; }
			if (itemAnimation < itemAnimationMax * 0.33f) { ai[0] -= moveInward; } else { ai[0] += moveOutward; }
			p.position += p.velocity * ai[0];
			if (!overrideKill && Main.player[p.owner].itemAnimation == 0) { p.Kill(); }
			p.rotation = p.velocity.ToRotation() + 2.355f;
			if (p.spriteDirection == -1) { p.rotation -= 1.57f; }
		}

		//draws the spear properly
		public static void DrawProjectileSpear(SpriteBatch sb, Texture2D texture, Projectile p, Color? overrideColor = null, float offsetX = 0f, float offsetY = 0f)
		{
			offsetX += (-texture.Width * 0.5f);
			Vector2 playerCenter = Main.player[p.owner].Center;
			Color lightColor = overrideColor != null ? (Color)overrideColor : p.GetAlpha(Lighting.GetColor((int)(playerCenter.X / 16f), (int)(playerCenter.Y / 16f)));
			Vector2 origin = new Vector2((float)texture.Width * 0.5f, (float)texture.Height * 0.5f);
			Vector2 offset = RotateVector(p.Center, p.Center + new Vector2(p.direction == -1 ? offsetX : offsetY, p.direction == 1 ? offsetX : offsetY), p.rotation - 2.355f) - p.Center;
			sb.Draw(texture, p.Center - Main.screenPosition + offset, new Rectangle(0, 0, texture.Width, texture.Height), lightColor, p.rotation, origin, p.scale, p.direction == -1 ? SpriteEffects.FlipHorizontally : SpriteEffects.None, 0f);
		}

		//rotates a vector around another vector by the rotation amount.
		public static Vector2 RotateVector(Vector2 origin, Vector2 vecToRot, float rot)
		{
			float newPosX = (float)(Math.Cos(rot) * (vecToRot.X - origin.X) - Math.Sin(rot) * (vecToRot.Y - origin.Y) + origin.X);
			float newPosY = (float)(Math.Sin(rot) * (vecToRot.X - origin.X) + Math.Cos(rot) * (vecToRot.Y - origin.Y) + origin.Y);
			return new Vector2(newPosX, newPosY);
		}
		#endregion

	}
}
